/**********************************************************

  The Gibbs Sampler for Fitting Overcomplete Linear Basis 

***********************************************************/



#include "mex.h"     /* the C code to be run in matlab */

#include <math.h>



/* generating random number uniform [0, 1] */

/* note #undef's at end of file */

#define IM1 2147483563

#define IM2 2147483399

#define AM (1.0/IM1)

#define IMM1 (IM1-1)

#define IA1 40014

#define IA2 40692

#define IQ1 53668

#define IQ2 52774

#define IR1 12211

#define IR2 3791

#define NTAB 32

#define NDIV (1+IMM1/NTAB)

#define EPS 1.2e-7

#define RNMX (1.0-EPS)

   

double ran2(long *idum)

{

	int j;

	long k;

	static long idum2=123456789;

	static long iy=0;

	static long iv[NTAB];

	double temp;

   

	if (*idum <= 0) {

		if (-(*idum) < 1) *idum=1;

		else *idum = -(*idum);

		idum2=(*idum);

		for (j=NTAB+7;j>=0;j--) {

			k=(*idum)/IQ1;



*idum=IA1*(*idum-k*IQ1)-k*IR1;

			if (*idum < 0) *idum += IM1;

			if (j < NTAB) iv[j] = *idum;

		}

		iy=iv[0];

	}

	k=(*idum)/IQ1;

	*idum=IA1*(*idum-k*IQ1)-k*IR1;

	if (*idum < 0) *idum += IM1;

	k=idum2/IQ2;

	idum2=IA2*(idum2-k*IQ2)-k*IR2;

	if (idum2 < 0) idum2 += IM2;

	j=iy/NDIV;

	iy=iv[j]-idum2;

	iv[j] = *idum;

	if (iy < 1) iy += IMM1;

	if ((temp=AM*iy) > RNMX) return RNMX;

	else return temp;

}

#undef IM1

#undef IM2

#undef AM

#undef IMM1

#undef IA1

#undef IA2

#undef IQ1

#undef IQ2

#undef IR1

#undef IR2

#undef NTAB

#undef NDIV



#undef EPS

#undef RNMX





/* Generating standard Gaussian distribution N(0, 1) */ 

double gasdev(long *idum)

{

	double ran2(long *idum);

	static int iset=0;

	static double gset;

	double fac,rsq,v1,v2;

   

	if (*idum < 0) iset=0;

	if  (iset == 0) {

		do {

			v1=2.0*ran2(idum)-1.0;

			v2=2.0*ran2(idum)-1.0;

			rsq=v1*v1+v2*v2;

		} while (rsq >= 1.0 || rsq == 0.0);

		fac=sqrt(-2.0*log(rsq)/rsq);

		gset=v1*fac;

		iset=1;

		return v2*fac;

	} else {

		iset=0;

		return gset;

	}

}



/* Compute the tail probability of normal distribution */

double erfcc(double x)

{

	double t,z,ans;

   

	z=fabs(x);

	t=1.0/(1.0+0.5*z);

	ans=t*exp(-z*z-1.26551223+t*(1.00002368+t*(0.37409196+t*(0.09678418+

		t*(-0.18628806+t*(0.27886807+t*(-1.13520398+t*(1.48851587+

		t*(-0.82215223+t*0.17087277)))))))));

	return x >= 0.0 ? ans : 2.0-ans;

}



/* This is Prob(Z>z), where Z ~ N(0, 1) */

double nf(double z)

{

   return erfcc(z/sqrt(2.0))/2.0; 

}



/* getting the index of (x, y) for image of size (sx, sy) */

int px(int x, int y, int sx, int sy)

{            

   return (x + (y-1)*sx - 1); 

 }



# define ABS(x) ((x)>0? (x):(-(x)))

# define MAX(x, y) ((x)>(y)? (x):(y))

# define MIN(x, y) ((x)<(y)? (x):(y))



void Cgibbs(double *I, int sx, int sy, 

         int N, double **Fker, int *Fhalf, int *Fsub, 

         double **fI, double **FI, double **C, int T, 

         double RESP, double TAUSQ, double RHO)

/* I: image to be fitted

   (sx, sy): size of image 

   N: # of filters

   Fker: filter kernels

   Fhalf: filter half-sizes

   Fsub: filter subsamplings

   fI: analysis coefficients of remainder image

   FI: synthesis coefficients

   C: correlation/inhibition

   T: total # of iterations for Gibbs sampler

   RESP: threshold for being responsive 

   TAUSQ: prior variance of responsive coefficients

   RHO: prior prob of being non-responsive */

{

   int ind, h, x, y, mi, mx, my, t0, t1, mh, gh; 

   int add=0, del=0,upd=0, tot=0; 

   double m, *f, *fc, a, sigsq = 1., sigsq_star, beta_star, BF, b; 

   long dum= 1234567, *idum; 



   idum = &dum;   /* initializing the random number generator */



   gh = Fhalf[N-1]; 

   for (t0=0; t0<T; t0++)

   {   

    sigsq_star = 1./(1./sigsq + 1./TAUSQ); 

    /* if (t0<3) sigsq = 1.; */



    add = 0; del = 0; upd = 0; 

    for (t1=0; t1<N*sx*sy; t1++) 

    /*for (mi=N-1; mi>=0; mi--)*/ 

     {

        mi = floor(ran2(idum)*N);  /* randomly pick a filter */      

        mh = Fhalf[mi]; 

        f = fI[mi];

    /* for (mx=mh+1; mx<=sx-mh; mx++)

      for (my=mh+1; my<=sy-mh; my++) */

      {

        /* randomly pick a location */

       mx = mh+1 + floor(ran2(idum)*(sx-mh-mh)); 

       my = mh+1 + floor(ran2(idum)*(sy-mh-mh)); 

         

        b = FI[mi][px(mx, my, sx, sy)]; 

        beta_star = sigsq_star/sigsq*

                    (f[px(mx, my, sx, sy)]+b);  /* posterior mean */



       /* Bayes factor of response versus non-response */

        BF = exp(beta_star*beta_star/2./sigsq_star)*

             sqrt(sigsq_star/TAUSQ)*

            (nf((RESP-beta_star)/sqrt(sigsq_star)) + 

              1.0 - nf((-RESP-beta_star)/sqrt(sigsq_star)))/

            (nf(RESP/sqrt(TAUSQ)) + 1.0 - nf(-RESP/sqrt(TAUSQ))); 

           

        if (ran2(idum)<(RHO/(RHO+(1.-RHO)*BF)))

           {

              /* non-response */

              a = 0.;

              if (b!=0.)

                 del ++; 

            }

         else 

           {

            /* response */

            do 

               a = beta_star + 

                  (double)(gasdev(idum)*sqrt(sigsq_star));

            while (ABS(a)<RESP);  

            if (b==0.)

                add ++; 

            else upd ++; 

           }

         FI[mi][px(mx, my, sx, sy)] = a;  

         a -=  b; 

         if (ABS(a)>1e-5)   

           {             

             /* updating analysis coefficients */

             for (ind=0; ind<N; ind++)

               {

                 h = Fhalf[ind]; 

                 f = fI[ind]; 

                 fc = C[mi+ind*N]; 

                 for (x=MAX(1, mx-mh-h); x<=MIN(sx, mx+mh+h); x++)

                   for (y=MAX(1, my-mh-h); y<=MIN(sy, my+mh+h); y++)

                       f[px(x, y, sx, sy)] -= 

                          a*fc[px(x-mx+mh+h+1, y-my+mh+h+1, 2*(mh+h)+1, 2*(mh+h)+1)];                  

                 } 

             /* updating remainder image */

             for (x=MAX(1, mx-mh); x<=MIN(sx, mx+mh); x++)

                   for (y=MAX(1, my-mh); y<=MIN(sy, my+mh); y++)

                      {

                        f = Fker[mi]; 

                        I[px(x, y, sx, sy)] -= 

                          a*f[px(x-mx+mh+1, y-my+mh+1, 2*mh+1, 2*mh+1)];

                      }

             } /* end if */

      } /*end for mx my */

   } /* end for mi */

   

tot += (add-del); 



/* compute variance */

sigsq = 0.;

for (x=gh+1; x<=sx-gh; x++)

   for (y=gh+1; y<=sy-gh; y++)

        {

          a = I[px(x, y, sx, sy)]; 

          sigsq += a*a; 

         }

 sigsq /= ((sx-gh-gh)*(sy-gh-gh)); 

 printf("add = %d   del = %d   upd = %d   tot = %d --> %f \n", 

         add,       del,       upd,       tot,         sigsq); 



}  /* end for t0 */

}





void mexFunction(int nlhs, mxArray *plhs[], 

                 int nrhs, const mxArray *prhs[])                

{

 double *I; 

 int sx, sy; 



 int N, ind, j, *Fhalf, *Fsub, T; 

 double **Fker, **fI, **FI, **C, a, RESP, TAUSQ, RHO; 

 mxArray *f, *g; 

 /* I: image to be fitted

   (sx, sy): size of image 

   N: # of filters

   Fker: filter kernels

   Fhalf: filter half-sizes

   Fsub: filter subsamplings

   fI: analysis coefficients of remainder image

   FI: synthesis coefficients

   C: correlation/inhibition

   T: total # of iterations for Gibbs sampler

   RESP: threshold for being responsive 

   TAUSQ: prior variance of responsive coefficients

   RHO: prior prob of being non-responsive */



 I = mxGetPr(prhs[0]); 

 sx = mxGetM(prhs[0]); 

 sy = mxGetN(prhs[0]); 

 

 N = mxGetNumberOfElements(prhs[1]); 

 Fker = mxCalloc(N, sizeof(double*)); 

 Fhalf = mxCalloc(N, sizeof(int)); 

 Fsub = mxCalloc(N, sizeof(int)); 

 

 FI = mxCalloc(N, sizeof(double*)); 

 fI = mxCalloc(N, sizeof(double*)); 

 

 C = mxCalloc(N*N, sizeof(double*)); 



 for (ind=0; ind<N; ind++)

     {  

       f = mxGetCell(prhs[1], ind); 

       g = mxGetField(f, 0, "ker"); 

       Fker[ind] = mxGetPr(g); 

       g = mxGetField(f, 0, "half"); 

       a = mxGetScalar(g); 

       Fhalf[ind] = (int)a; 

       g = mxGetField(f, 0, "sub");

       a = mxGetScalar(g); 

       Fsub[ind] = (int)a; 

       f = mxGetCell(prhs[2], ind); 

       fI[ind] = mxGetPr(f); 

       f = mxGetCell(prhs[3], ind); 

       FI[ind] = mxGetPr(f); 

       for (j=0; j<N; j++)

        {

         f = mxGetCell(prhs[4], j*N+ind); 

         C[j*N+ind] = mxGetPr(f); 

        }

    }

a = mxGetScalar(prhs[5]);  

T = (int)a; 

RESP = mxGetScalar(prhs[6]); 

TAUSQ = mxGetScalar(prhs[7]); 

RHO = mxGetScalar(prhs[8]); 



printf("%d   %f   %f   %f \n", T, RESP, TAUSQ, RHO); 



Cgibbs(I, sx, sy, 

       N, Fker, Fhalf, Fsub, fI, FI, C, T, 

       RESP, TAUSQ, RHO); 

}

      



 

              