/******************************************

  The Match Pursuit Algorithm of Mallat

*******************************************/



# include "mex.h"        /* the algorithm is connect to matlab */

# include "math.h"

# define ABS(x) ((x)>0? (x):(-(x)))

# define MAX(x, y) ((x)>(y)? (x):(y))

# define MIN(x, y) ((x)<(y)? (x):(y))



/* getting the index of matlab image, (x,y) location, (sx,sy) sizes */

int px(int x, int y, int sx, int sy)    

{            

   return (x + (y-1)*sx - 1); 

 }



/* the match pursuit algorithm */

void Cmp(double *I, int sx, int sy, 

         int N, double **Fker, int *Fhalf, int *Fsub, 

         double **fI, double **FI, double **C, double Tresh)

/* I: image to be analyzed

   (sx, sy): sizes of I

   N: number of filters

   Fker: the kernels of the filters

   Fhalf: the half-sizes of filters

   Fsub: subsampling rate of filters

   fI: analysis coefficients of filters on remainder image

   FI: synthesis coefficients of filters

   C: correlation/inhibition coefficients

   Tresh: Threshold for stopping criterion  */

{

   int ind, h, x, y, mi, mx, my, t, mh; 

   double m, *f, *fc, a; 



  t = 0; 

  do  /* find the maximum analysis coefficient */

   {

     m = 0.; 

     for (ind=0; ind<N; ind++)

     {

        h = Fhalf[ind]; 

        f = fI[ind]; 

        for (x=h+1; x<=sx-h; x++)

           for (y=h+1; y<=sy-h; y++)

              if (m<ABS(f[px(x, y, sx, sy)]))

                 {

                  m = ABS(f[px(x, y, sx, sy)]); 

                  mi = ind; mx = x; my = y; 

                 }

      }

     a = fI[mi][px(mx, my, sx, sy)]; 

     FI[mi][px(mx, my, sx, sy)] += a;  /* update synthesis coefficient */

     mh = Fhalf[mi];  

     t++; 

     printf("%d:  %d (%d %d) %f\n", t, mi, mx, my, a); 



     for (ind=0; ind<N; ind++)  /* update analysis coefficients */

       {

        h = Fhalf[ind]; 

        f = fI[ind]; 

        fc = C[mi+ind*N]; 

        for (x=MAX(1, mx-mh-h); x<=MIN(sx, mx+mh+h); x++)

          for (y=MAX(1, my-mh-h); y<=MIN(sy, my+mh+h); y++)

            f[px(x, y, sx, sy)] -= 

            a*fc[px(x-mx+mh+h+1, y-my+mh+h+1, 2*(mh+h)+1, 2*(mh+h)+1)];                  

      } 

   }

   while (ABS(a)>Tresh);   /* stopping criterion */

}





/* the following is the mex file for communicating with matlab */



void mexFunction(int nlhs, mxArray *plhs[], 

                 int nrhs, const mxArray *prhs[])                

{

 double *I;            /* image I */

 int sx, sy;           /* sizes of image */ 



 int N, ind, j, *Fhalf, *Fsub;      

 double **Fker, **fI, **FI, **C, a, Tresh; 

 mxArray *f, *g; 

/* I: image to be analyzed

   (sx, sy): sizes of I

   N: number of filters

   Fker: the kernels of the filters

   Fhalf: the half-sizes of filters

   Fsub: subsampling rate of filters

   fI: analysis coefficients of filters on remainder image

   FI: synthesis coefficients of filters

   C: correlation/inhibition coefficients

   Tresh: Threshold for stopping criterion  

   f, g: temporary variables  */



 I = mxGetPr(prhs[0]);    /* get the matlab pointer */

 sx = mxGetM(prhs[0]);    /* get size of x-direction */

 sy = mxGetN(prhs[0]);    /* get size of y-direction */

 

 N = mxGetNumberOfElements(prhs[1]);  /* get total number of filters */

 Fker = mxCalloc(N, sizeof(double*)); /* Fker: kernels of filters */

 Fhalf = mxCalloc(N, sizeof(int));    /* Fhalf: half-sizes of filters */

 Fsub = mxCalloc(N, sizeof(int));     /* Fsub: subsampling rate of filters */

 

 FI = mxCalloc(N, sizeof(double*));   /* FI: synthesis coefficients */

 fI = mxCalloc(N, sizeof(double*));   /* fI: analysis coefficients of remainder */

 

 C = mxCalloc(N*N, sizeof(double*));  /* C: correlation/inhibition between filters */



 for (ind=0; ind<N; ind++)

     {  

       f = mxGetCell(prhs[1], ind); 

       g = mxGetField(f, 0, "ker"); 

       Fker[ind] = mxGetPr(g);           /* get kernels */

       g = mxGetField(f, 0, "half"); 

       a = mxGetScalar(g);             

       Fhalf[ind] = (int)a;              /* get half-size */

       g = mxGetField(f, 0, "sub");

       a = mxGetScalar(g); 

       Fsub[ind] = (int)a;                /* get subsampling */

       f = mxGetCell(prhs[2], ind); 

       fI[ind] = mxGetPr(f);              /* get analysis coefficients */

       f = mxGetCell(prhs[3], ind); 

       FI[ind] = mxGetPr(f);              /* get synthesis coefficients */

       for (j=0; j<N; j++)

        {

         f = mxGetCell(prhs[4], j*N+ind); 

         C[j*N+ind] = mxGetPr(f);         /* get correlation/inhibition */

        }

    }

Tresh = mxGetScalar(prhs[5]);             /* get threshold */ 

printf("%f \n", Tresh); 



/* call enginee */ 

Cmp(I, sx, sy, 

    N, Fker, Fhalf, Fsub, 

    fI, FI, C, Tresh); 

}

      



 

                    