/**********************************************

    Synthesize images from coefficients bands

***********************************************/



# include "mex.h"        /* this is a mex C routine for matlab */

# include "math.h"



/* get index for image intensities, (x, y) pixel, (sx, sy) sizes */

int px(int x, int y, int sx, int sy)

{

   if (x<1) x += sx; 

   if (x>sx) x -= sx; 

   if (y<1) y += sy;

   if (y>sy) y -= sy; 

            

   return (x + (y-1)*sx - 1); 

 }



/* synthesize the image */

void Csynthesize(double *I, int sx, int sy, 

                 int N, double **Fker, int *Fhalf, int *Fsub, 

                 double **FI)

/* I: image to be synthesized 

   (sx, sy): sizes of image along x and y directions

   N: number of filters

   Fker: kernels of filters

   Fhalf: half-sizes of filters

   Fsub: subsampling rates of filters

   FI: synthesis coefficients */

{

   int ind, x, y, fx, fy, h, s, sxi, syi; 

   double coef, *f, *fi; 

 

   for (ind=0; ind<N; ind++)

      {

         h = Fhalf[ind]; s = Fsub[ind]; f = Fker[ind];

         sxi = (int)(sx/s); syi = (int)(sy/s); 

     

         fi = FI[ind]; 

       

         for (x=1; x<=sxi; x++)

            for (y=1; y<=syi; y++)

               {

                  coef = fi[px(x, y, sxi, syi)]; 

                  if (coef!=0.)

                    for (fx=1; fx<=h+h+1; fx++)

                       for (fy=1; fy<=h+h+1; fy++)

                          I[px(x*s + fx-h-1, y*s + fy-h-1, sx, sy)] +=

                              coef*f[px(fx, fy, h+h+1, h+h+1)];                             

               }                              

      }      

}



/* mex function for getting the data/pointers from matlab */

void mexFunction(int nlhs, mxArray *plhs[], 

                 int nrhs, const mxArray *prhs[])                

{

 double *I; 

 int sx, sy; 



 int N, ind, *Fhalf, *Fsub; 

 double **Fker, **FI, a; 

 mxArray *f, *g; 



 /* I: image to be synthesized 

   (sx, sy): sizes of image along x and y directions

   N: number of filters

   Fker: kernels of filters

   Fhalf: half-sizes of filters

   Fsub: subsampling rates of filters

   FI: synthesis coefficients */



 I = mxGetPr(prhs[0]);     /* get pointer of image */

 sx = mxGetM(prhs[0]);     /* get x size */

 sy = mxGetN(prhs[0]);     /* get y size */

 

 N = mxGetNumberOfElements(prhs[1]);   /* get total # of filters */

 Fker = mxCalloc(N, sizeof(double*));  /* filter kernels */

 Fhalf = mxCalloc(N, sizeof(int));     /* filter half-sizes */

 Fsub = mxCalloc(N, sizeof(int));      /* filter subsampling */

 

 FI = mxCalloc(N, sizeof(double*));    /* get synthesis subbands */



 for (ind=0; ind<N; ind++)

     {  

       f = mxGetCell(prhs[1], ind); 

       g = mxGetField(f, 0, "ker"); 

       Fker[ind] = mxGetPr(g);         /* get filter kernels */

       g = mxGetField(f, 0, "half"); 

       a = mxGetScalar(g);  

       Fhalf[ind] = (int)a;            /* get filter half-sizes */

       g = mxGetField(f, 0, "sub");

       a = mxGetScalar(g); 

       Fsub[ind] = (int)a;             /* get filter subsamplings */

       f = mxGetCell(prhs[2], ind); 

       FI[ind] = mxGetPr(f);           /* get synthesis subbands */

    }

/* call C function to synthesize */

Csynthesize(I, sx, sy, 

            N, Fker, Fhalf, Fsub, 

            FI); 

}

      



 

                    