function [B,C] = doog(sigma, orientation, aspect)

% [B,C] = doog(sigma, orientation, aspect)

% Even DOOG base, i.e., second derivative 

% sigma: scale of the Gaussian in x-direction 

% orientation: in degree

% aspect ratio: y:x

% B: the base matrix

% C: the symbolic representation 



size = sigma * 4 * aspect + 1;     % size of the base

c = sigma * 2 * aspect + 1;        % half-size

theta = (pi * orientation)/180;    % orientation in real number



B = zeros(size);      % the base matrix

r = aspect;           % aspect ratio

for i = 1:size

  for j = 1:size

    x = ((i-c) * cos(theta) + (j-c) * sin(theta))/sigma;

    y = ((c-i) * sin(theta) + (j-c) * cos(theta))/sigma;

                      % rotate and scale x and y

    B(i, j) = (2*G(x, 1)-G(x-1, 1)-G(x+1, 1))*G(y, r); 

                      % second derivative 

 end

end



tmp = B(:); 

B = B/sqrt(sum(tmp.*tmp));    % normalized to norm 1



if (rem(size, 2)==0)          % the size of B should be odd 

   B1 = zeros(size+1); 

   B1(1:size, 1:size) = B;

   B = B1; 

end



C = B-B;           % symbolic representation 

for i = 1:size

  for j = 1:size

    x = ((i-c) * cos(theta) + (j-c) * sin(theta));

    y = ((c-i) * sin(theta) + (j-c) * cos(theta));

    if ((abs(x)<1.2)&(abs(y/sigma)<1.5*aspect))

       C(i, j) = 1;      % a thin bar 

    end

 end

end





