function C = corr(F)

% C = corr(F)

%---------------------------------------------------------

% Compute correlation or inhibition coefficients so that 

% after the synthesis coefficient of a base is updated, 

% the analysis coefficients of all the overlapping bases 

% can be updated without computing the remainder image. 

%---------------------------------------------------------

% F: the bank of filters

% C: the correlation matrix



N = size(F, 2);   % N: the number of filters



for (i = 1:N) 

   hi = F{i}.half;   % half-size of filter i

   for (j = 1:N)

      hj = F{j}.half;    % half-size of filter j

      I = zeros(2*(hi+hj)+1);   

      I((hj+1):(hj+2*hi+1), (hj+1):(hj+2*hi+1)) = F{i}.ker; 

      % I is the image patch of filter i at (0, 0)

      % the domain of I collects all positions of filter j

      % that can overlap with filter i at (0, 0)

      C{i, j} = filter2(F{j}.ker, I, 'same'); 

      % C{i,j} is how filter i at (0,0) inhibites filter j at 

      % locations overlapping with filter i

   end

end

