function FI = gibbs(I, F, C, T)

% FI = gibbs(I, F, C, T)

% ----------------------------------------------------------

% Fitting the linear sysnthesis model by Gibbs sampling

% The model is I = sum r_i f_i + N(0, sigsq)  

% The prior is with prob = RHO, r_i = 0; 

% with prob = 1-RHO, r_i ~ N(0, TAUSQ) with |r_i|>RESP

% -----------------------------------------------------------

% I: the image to be fitted 

% F: the filter bank

% C: the correlation/inhibition coefficients

% T: number of iterations for the Gibbs sampling 

% FI: the synthesis coefficients



RESP = 3.;       % threshold for a cell to be responsive 

TAUSQ = 100.;    % the variance of the prior distribution 

RHO = .9999;     % the prior prob of non-response 



N = size(F, 2);  % the total number of filters

M = size(I);     % the size of image 

J = I + 0;       % we are going to update J in C function  

for (i = 1:N)

   FI{i} = zeros(M);  % FI: the synthesis coefficients 

   fI{i} = filter2(F{i}.ker, J, 'same'); 

                  % fI: the analysis coefficients of remainder image 

end



% call mex C function 

Cgibbs(J, F, fI, FI, C, T, RESP, TAUSQ, RHO); 







      